@@ -3,6 +3,9 @@
 #include <sys/types.h>
 #include <utmp.h>
 #include <string.h>
+
+#include "sys_defs.h"
+
 #ifdef HAS_UT_ADDR
 #include <netdb.h>
 #endif
@@ -34,7 +37,9 @@ char   *host;
 	  || ut->ut_type == LOGIN_PROCESS || ut->ut_type == USER_PROCESS)) {
 	    strncpy(ut->ut_line, line, sizeof(ut->ut_line));
 	    strncpy(ut->ut_user, user, sizeof(ut->ut_user));
+#ifndef NO_UT_HOST
 	    strncpy(ut->ut_host, host, sizeof(ut->ut_host));
+#endif
 	    time(&(ut->ut_time));
 #ifdef HAS_UT_ADDR
 	    if ((hp = gethostbyname(host)) != 0)
