@@ -45,6 +45,10 @@ static char sccsid[] = "@(#)login.c	5.32.1.1 (Berkeley) 1/28/89";
 #include <sys/file.h>
 #include <termios.h>
 
+#ifndef MAXHOSTNAMELEN
+#include <netdb.h>	/* SunOS 5.x... */
+#endif
+
 #ifdef HAS_UTMPX
 #include <utmpx.h>
 #else /* HAS_UTMPX */
@@ -138,7 +142,7 @@ char	*checklat();
 #define KEY
 #endif /* SNK */
 
-static	char rusername[100], lusername[100];
+static	char rusername[100], lusername[100], wherefrom[MAXHOSTNAMELEN + 108];
 
 /*
  * This bounds the time given to login.  Not a define so it can
@@ -186,7 +190,7 @@ main(argc, argv)
 	register char *p;
 	int ask, fflag, hflag, pflag, cnt;
 	int rflag;
-	int quietlog, passwd_req, ioctlval;
+	int quietlog, passwd_req;
 	void timedout();
 	int last_fd;
 	void hungup();
@@ -356,6 +360,17 @@ main(argc, argv)
 	} else
 		ask = 1;
 
+	/*
+	 * When HPUX can't find the hostname it passes the server address
+	 * on the command line...
+	 */
+#ifdef HPUX9
+	if (hostname && hostname[strspn(hostname, "0123456789.")] == 0) {
+		char *utmp_host();
+		hostname = utmp_host();
+	}
+#endif
+
 	/*
 	 * Finalize the terminal settings. Some systems default to 8 bits,
 	 * others to 7, so we should leave that alone.
@@ -390,6 +405,14 @@ main(argc, argv)
 	else
 		tty = ttyn;
 
+	/* Fill in wherefrom if it hasn't been done yet */
+	if (wherefrom[0] == '\0') {
+		if (hostname)
+			sprintf(wherefrom, "from %s", hostname);
+		else
+			sprintf(wherefrom, "on %s", tty);
+	}
+
 	for (cnt = 0;; ask = 1) {
 
 		if (ask) {
@@ -486,11 +509,7 @@ main(argc, argv)
 	 * this terminal are permitted.
 	 */
 	if (pwd->pw_uid == 0 && !rootterm(tty)) {
-		if (hostname)
-			syslog(LOG_NOTICE, "ROOT LOGIN REFUSED FROM %s",
-			    hostname);
-		else
-			syslog(LOG_NOTICE, "ROOT LOGIN REFUSED ON %s", tty);
+		syslog(LOG_NOTICE, "ROOT LOGIN REFUSED %s", wherefrom);
 		printf("Login incorrect\n");
 		sleepexit(1);
 	}
@@ -499,11 +518,7 @@ main(argc, argv)
 	 * Syslog each successful login, so we don't have to watch hundreds
 	 * of wtmp or lastlogin files.
 	 */
-	if (hostname) {
-		syslog(LOG_INFO, "login from %s as %s", hostname, pwd->pw_name);
-	} else {
-		syslog(LOG_INFO, "login on %s as %s", tty, pwd->pw_name);
-	}
+	syslog(LOG_INFO, "login %s as %s", wherefrom, pwd->pw_name);
 
 	/*
 	 * Update the utmp files, either BSD or SYSV style.
@@ -523,6 +538,9 @@ main(argc, argv)
 		if (hostname)
 			strncpy(utmp.ut_host, hostname, sizeof(utmp.ut_host));
 		strncpy(utmp.ut_line, tty, sizeof(utmp.ut_line));
+#ifdef USER_PROCESS
+		utmp.ut_type = USER_PROCESS;
+#endif
 		login(&utmp);
 	}
 #endif /* SYSV_UTMP */
@@ -680,11 +698,7 @@ main(argc, argv)
 #endif /* WIN */
 
 	if (pwd->pw_uid == 0)
-		if (hostname)
-			syslog(LOG_NOTICE, "ROOT LOGIN ON %s FROM %s",
-			    tty, hostname);
-		else
-			syslog(LOG_NOTICE, "ROOT LOGIN ON %s", tty);
+		syslog(LOG_NOTICE, "ROOT LOGIN %s", wherefrom);
 
 #ifndef NO_MOTD
 	/*
@@ -698,7 +712,11 @@ main(argc, argv)
 #ifdef WIN /* Mangled mailpath */
 		strcpy(tbuf, mailpath);
 #else /* WIN */
+#ifdef HOMEDOTMAIL /* Mail in ~/.mail */
+		(void)sprintf(tbuf, "%s/.mail", pwd->pw_dir);
+#else /* HOMEDOTMAIL */
 		(void)sprintf(tbuf, "%s/%s", _PATH_MAILDIR, pwd->pw_name);
+#endif /* HOMEDOTMAIL */
 #endif /* WIN */
 		if (stat(tbuf, &st) == 0 && st.st_size != 0)
 			printf("You have %smail.\n",
@@ -712,14 +730,10 @@ main(argc, argv)
 	 * What complicates matters is that S/Key may also used for other
 	 * utilities that ask passwords, such as ftpd, rexecd or su.
 	 */
-	if (login_access(pwd->pw_name, hostname ? hostname : tty) == 0) {
+	if (login_access(pwd, hostname ? hostname : tty) == 0) {
 		printf("Permission denied\n");
-		if (hostname)
-			syslog(LOG_NOTICE, "%s LOGIN REFUSED FROM %s",
-			    pwd->pw_name, hostname);
-		else
-			syslog(LOG_NOTICE, "%s LOGIN REFUSED ON %s", 
-			    pwd->pw_name, tty);
+		syslog(LOG_NOTICE, "%s LOGIN REFUSED %s",
+			    pwd->pw_name, wherefrom);
 		sleepexit(1);
 	}
 
@@ -944,12 +958,8 @@ badlogin(name)
 {
 	if (failures < (pwd ? 1 : 2))
 		return;
-	if (hostname)
-		syslog(LOG_NOTICE, "%d LOGIN FAILURE%s FROM %s, %s",
-		    failures, failures > 1 ? "S" : "", hostname, name);
-	else
-		syslog(LOG_NOTICE, "%d LOGIN FAILURE%s ON %s, %s",
-		    failures, failures > 1 ? "S" : "", tty, name);
+	syslog(LOG_NOTICE, "%d LOGIN FAILURE%s %s, %s",
+		failures, failures > 1 ? "S" : "", wherefrom, name);
 }
 
 #ifndef NO_TTYENT
@@ -975,6 +985,7 @@ doremotelogin(host)
 	getstr(rusername, sizeof (rusername), "remuser");
 	getstr(lusername, sizeof (lusername), "locuser");
 	getstr(term, sizeof(term), "Terminal type");
+	sprintf(wherefrom, "from %s@%s", rusername, host);
 	pwd = getpwnam(lusername);
 	if (pwd == NULL)
 		return(-1);
